--- old/usr/src/uts/i86pc/os/fakebop.c	Tue Jan 21 15:29:48 2020
+++ new/usr/src/uts/i86pc/os/fakebop.c	Tue Jan 21 15:29:48 2020
@@ -101,7 +101,7 @@
 
 /*
  * Boot info from "glue" code in low memory. xbootp is used by:
- *	do_bop_phys_alloc(), do_bsys_alloc() and boot_prop_finish().
+ *	do_bop_phys_alloc(), do_bsys_alloc() and read_bootenvrc().
  */
 static struct xboot_info *xbootp;
 static uintptr_t next_virt;	/* next available virtual address */
@@ -670,7 +670,7 @@
  * we do single character I/O since this is really just looking at memory
  */
 void
-boot_prop_finish(void)
+read_bootenvrc(void)
 {
 	int fd;
 	char *line;
@@ -778,17 +778,32 @@
 
 		/*
 		 * If a property was explicitly set on the command line
-		 * it will override a setting in bootenv.rc
+		 * it will override a setting in bootenv.rc. We make an
+		 * exception for a property from the bootloader such as:
+		 *
+		 * console="text,ttya,ttyb,ttyc,ttyd"
+		 *
+		 * In such a case, picking the first value here (as
+		 * lookup_console_devices() does) is at best a guess; if
+		 * bootenv.rc has a value, it's probably better.
 		 */
-		if (do_bsys_getproplen(NULL, name) >= 0)
+		if (strcmp(name, "console") == 0) {
+			char propval[BP_MAX_STRLEN] = "";
+
+			if (do_bsys_getprop(NULL, name, propval) == -1 ||
+			    strchr(propval, ',') != NULL)
+				bsetprops(name, value);
 			continue;
+		}
 
-		bsetprops(name, value);
+		if (do_bsys_getproplen(NULL, name) == -1)
+			bsetprops(name, value);
 	}
 done:
 	if (fd >= 0)
 		(void) BRD_CLOSE(bfs_ops, fd);
 
+
 	/*
 	 * Check if we have to limit the boot time allocator
 	 */
@@ -843,7 +858,7 @@
 			v_len = 0;
 		}
 		consoledev[v_len] = 0;
-		bcons_init2(inputdev, outputdev, consoledev);
+		bcons_post_bootenvrc(inputdev, outputdev, consoledev);
 	} else {
 		/*
 		 * Ensure console property exists
@@ -853,7 +868,7 @@
 		if (v_len < 0)
 			bsetprops("console", "hypervisor");
 		inputdev = outputdev = consoledev = "hypervisor";
-		bcons_init2(inputdev, outputdev, consoledev);
+		bcons_post_bootenvrc(inputdev, outputdev, consoledev);
 	}
 
 	if (find_boot_prop("prom_debug") || kbm_debug)
